@@ -32,6 +32,11 @@
 #include "vinagre-rdp-connection.h"
 #include "vinagre-rdp-tab.h"
 
+#define DEFAULT_WIDTH   800
+#define DEFAULT_HEIGHT  600
+#define MIN_SIZE          1
+#define MAX_SIZE       8192
+
 void rdp_register_types (void);
 static void vinagre_rdp_protocol_iface_init (VinagreProtocolInterface *iface);
 
@@ -89,28 +94,30 @@ vinagre_rdp_plugin_init (VinagreRdpPlugin *plugin)
 static GtkWidget *
 impl_get_connect_widget (VinagreProtocol *plugin, VinagreConnection *conn)
 {
-  GtkWidget *box, *label, *u_box, *u_entry;
+  GtkWidget *grid, *label, *u_entry, *spin_button;
   gchar     *str;
 
-  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
 
   str = g_strdup_printf ("<b>%s</b>", _("RDP Options"));
   label = gtk_label_new (str);
   g_free (str);
   gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-  gtk_container_add (GTK_CONTAINER (box), label);
-
-  u_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
 
   label = gtk_label_new_with_mnemonic (_("_Username:"));
-  gtk_box_pack_start (GTK_BOX (u_box), label, FALSE, FALSE, 0);
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
+  gtk_widget_set_margin_left (label, 12);
 
   u_entry = gtk_entry_new ();
   /* Translators: This is the tooltip for the username field in a RDP connection */
   gtk_widget_set_tooltip_text (u_entry, _("Optional. If blank, your username will be used. Also, it can be supplied in the Host field above, in the form username@hostname."));
-  g_object_set_data (G_OBJECT (box), "username_entry", u_entry);
-  gtk_container_add (GTK_CONTAINER (u_box), u_entry);
+  g_object_set_data (G_OBJECT (grid), "username_entry", u_entry);
+  gtk_grid_attach (GTK_GRID (grid), u_entry, 1, 1, 1, 1);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), u_entry);
   str = g_strdup (VINAGRE_IS_CONNECTION (conn) ?
 		  vinagre_connection_get_username (conn) :
@@ -119,9 +126,40 @@ impl_get_connect_widget (VinagreProtocol *plugin, VinagreConnection *conn)
   gtk_entry_set_activates_default (GTK_ENTRY (u_entry), TRUE);
   g_free (str);
 
-  gtk_widget_set_margin_left (u_box, 12);
-  gtk_container_add (GTK_CONTAINER (box), u_box);
-  return box;
+
+  /* Host width */
+  label = gtk_label_new_with_mnemonic (_("_Width:"));
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 2, 1, 1);
+  gtk_widget_set_margin_left (label, 12);
+
+  spin_button = gtk_spin_button_new_with_range (MIN_SIZE, MAX_SIZE, 1);
+  /* Translators: This is the tooltip for the width field in a RDP connection */
+  gtk_widget_set_tooltip_text (spin_button, _("Set width of the remote desktop"));
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_button), DEFAULT_WIDTH);
+  g_object_set_data (G_OBJECT (grid), "width_spin_button", spin_button);
+  gtk_grid_attach (GTK_GRID (grid), spin_button, 1, 2, 1, 1);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_button);
+  gtk_entry_set_activates_default (GTK_ENTRY (spin_button), TRUE);
+
+
+  /* Host height */
+  label = gtk_label_new_with_mnemonic (_("_Height:"));
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 3, 1, 1);
+  gtk_widget_set_margin_left (label, 12);
+
+  spin_button = gtk_spin_button_new_with_range (MIN_SIZE, MAX_SIZE, 1);
+  /* Translators: This is the tooltip for the height field in a RDP connection */
+  gtk_widget_set_tooltip_text (spin_button, _("Set height of the remote desktop"));
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_button), DEFAULT_HEIGHT);
+  g_object_set_data (G_OBJECT (grid), "height_spin_button", spin_button);
+  gtk_grid_attach (GTK_GRID (grid), spin_button, 1, 3, 1, 1);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_button);
+  gtk_entry_set_activates_default (GTK_ENTRY (spin_button), TRUE);
+
+
+  return grid;
 }
 
 static void
